#
# Copyright 2011-2013 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Refer to the README and COPYING files for full details of the license
#

import os
import textwrap
import ConfigParser

parameters = [
    # Section: [vars]
    ('vars', [

        ('core_dump_enable', 'true',
            'Enable core dump.'),

        ('profile_enable', 'false',
            'Enable whole process profiling (requires yappi profiler).'),

        ('profile_format', 'pstat',
            'Profile file format (pstat, callgrind, ystat)'),

        ('profile_builtins', 'false',
            'Profile builtin functions used by standard Python modules. '
            'false by default.'),

        ('profile_clock', 'cpu',
            'Sets the underlying clock type (cpu, wall)'),

        ('host_mem_reserve', '256',
            'Reserves memory for the host to prevent VMs from using all the '
            'physical pages. The values are in Mbytes.'),

        ('guest_ram_overhead', '65', None),

        ('extra_mem_reserve', '65',
            'Memory reserved for non-vds-administered programs.'),

        ('fake_nics', 'dummy_*,veth_*',
            'Comma-separated list of fnmatch-patterns for dummy hosts nics to '
            'be shown to vdsm.'),

        ('net_configurator', 'ifcfg',
            'Whether to use "ifcfg" or "iproute2" to configure networks. '
            'iproute2 configurator is not ready yet for genral usage.'),

        ('net_persistence', 'unified',
            'Whether to use "ifcfg" or "unified" persistence for networks.'),

        ('hwaddr_in_ifcfg', 'always',
            'Whether to set HWADDR in ifcfg files. Set to "never" if '
            'NetworkManager is disabled and device name persistence does '
            'not depend on HWADDR.'),

        ('ethtool_opts', '',
            'Which special ethtool options should be applied to NICs after '
            'they are taken up, e.g. "lro off" on buggy devices. '
            'To apply options to a single interface, '
            'set ethtool_opts.iface_name.'),

        ('nic_model', 'rtl8139,pv',
            'NIC model is rtl8139, ne2k_pci pv or any other valid device '
            'recognized by kvm/qemu if a coma separated list given then a '
            'NIC per device will be created.'),

        ('migration_destination_timeout', '21600',
            'Maximum time the destination waits for the migration to finish.'),

        ('migration_progress_timeout', '150',
            'Maximum time the source host waits during a migration in case '
            'that there is no progress. If the time has passed, the migration '
            'will be aborted.'),

        ('migration_max_time_per_gib_mem', '64',
            'The maximum time in seconds per GiB memory a migration may take '
            'before the migration will be aborted by the source host. '
            'Setting this value to 0 will disable this feature.'),

        ('migration_listener_timeout', '30',
            'Time to wait (in seconds) for migration destination to start '
            'listening before migration begins.'),

        ('migration_max_bandwidth', '32',
            'Maximum bandwidth for migration, in MiBps, 0 means libvirt\'s '
            'default, since 0.10.x default in libvirt is unlimited'),

        ('migration_monitor_interval', '10',
            'How often (in seconds) should the monitor thread pulse, 0 means '
            'the thread is disabled.'),

        ('hidden_nics', 'w*,usb*',
            'Comma-separated list of fnmatch-patterns for host nics to be '
            'hidden from vdsm.'),

        ('hidden_bonds', '',
            'Comma-separated list of fnmatch-patterns for host bonds to be '
            'hidden from vdsm.'),

        ('hidden_vlans', '',
            'Comma-separated list of fnmatch-patterns for host vlans to be '
            'hidden from vdsm. vlan names must be in the format "dev.VLANID" '
            '(e.g. eth0.100, em1.20, eth2.200). '
            'vlans with alternative names must be hidden from vdsm '
            '(e.g. eth0.10-fcoe, em1.myvlan100, vlan200)'),

        ('default_bridge', 'engine', None),

        ('migration_downtime', '500',
            'Maxmium allowed downtime for live migration in milliseconds '
            '(anything below 100ms is ignored) if you do not care about '
            'liveness of migration, set to a very high value, such as '
            '600000.'),

        ('migration_downtime_delay', '75',
            'This value is used on the source host to define the delay before '
            'setting/increasing the downtime of a migration. '
            'The value is per GiB of RAM. A minimum of twice this value is '
            'used for VMs with less than 2 GiB of RAM'),

        ('migration_downtime_steps', '10',
            'Incremental steps used to reach migration_downtime.'),

        ('max_outgoing_migrations', '3',
            'Maximum concurrent outgoing migrations'),

        ('sys_shutdown_timeout', '120',
            'Destroy and shutdown timeouts (in sec) before completing the '
            'action.'),

        ('user_shutdown_timeout', '30',
            'Grace period (seconds) to let guest user close his '
            'applications before shutdown.'),

        ('guest_agent_timeout', '30',
            'Time (in sec) to wait for guest agent.'),

        ('vm_command_timeout', '60',
            'Time to wait (in seconds) for vm to respond to a monitor '
            'command, 30 secs is a nice default. Set to 300 if the vm is '
            'expected to freeze during cluster failover.'),

        ('vm_watermark_interval', '2',
            'How often should we sample each vm for statistics (seconds).'),

        ('vm_sample_cpu_interval', '15', None),

        ('vm_sample_cpu_window', '2', None),

        ('vm_sample_disk_interval', '60', None),

        ('vm_sample_disk_window', '2', None),

        ('vm_sample_disk_latency_interval', '60', None),

        ('vm_sample_disk_latency_window', '2', None),

        ('vm_sample_net_interval', '15', None),

        ('vm_sample_net_window', '2', None),

        ('vm_sample_balloon_interval', '15', None),

        ('vm_sample_balloon_window', '2', None),

        ('vm_sample_jobs_interval', '15', None),

        # TODO: Change this to 1 once AdvancedStatsFunction can support it
        ('vm_sample_jobs_window', '2', None),

        ('vm_sample_vcpu_pin_interval', '15',
            'How often should we sample each vcpu runtime pinning to '
            'which physical cpu core.'),

        ('vm_sample_vcpu_pin_window', '2', None),

        ('vm_sample_cpu_tune_interval', '15', None),

        ('vm_sample_numa_interval', '15',
            'How often should we sample NUMA CPU assignments'),

        ('trust_store_path', '/etc/pki/vdsm',
            'Where the certificates and keys are situated.'),

        ('ssl', 'true',
            'Whether to use ssl encryption and authentication.'),

        ('vds_responsiveness_timeout', '60', None),

        ('vdsm_nice', '-5', None),

        ('qemu_drive_cache', 'none', None),

        ('fake_kvm_support', 'false', None),

        ('fake_kvm_architecture', 'x86_64',
            'Choose the target architecture of the fake KVM mode'),

        ('xmlrpc_enable', 'true', 'Enable the xmlrpc server'),

        ('xmlrpc_http11', 'true',
            'Enable HTTP/1.1 keep-alive connections'),

        ('jsonrpc_enable', 'true', 'Enable the JSON RPC server'),

        ('report_host_threads_as_cores', 'false',
            'Count each cpu hyperthread as an individual core'),

        ('libvirt_env_variable_log_filters', '',
            'Specify the log filters to track libvirt calls'),

        ('libvirt_env_variable_log_outputs', '',
            'Specify the output to track libvirt calls'),

        ('transient_disks_repository', '/var/lib/vdsm/transient',
            'Local path to the transient disks repository.'),
    ]),

    # Section: [ksm]
    ('ksm', [

        ('ksm_monitor_thread', 'true', None),

    ]),

    # Section: [mom]
    ('mom', [

        ('conf', '/etc/vdsm/mom.conf', 'mom configuration file'),

        ('tuning_policy', '01-parameters',
            'name of the mom policy to be updated from '
            'updatePolicyParameters API call'),

    ]),

    # Section: [irs]
    ('irs', [

        ('irs_enable', 'true', None),

        ('repository', '/rhev/data-center',
            'Image repository.'),

        ('hsm_tasks', '%(repository)s/hsm-tasks', None),

        ('images', '/images', None),

        ('irsd', '%(images)s/irsd', None),

        ('volume_utilization_percent', '50', None),

        ('volume_utilization_chunk_mb', '1024', None),

        ('vol_size_sample_interval', '60',
            'How often should the volume size be checked (seconds).'),

        ('scsi_rescan_maximal_timeout', '30',
            'The maximal number of seconds to wait for scsi scan to return.'),

        ('scsi_settle_timeout', '5',
            'Maximum number of seconds to wait until udev events are handled '
            'after modifying scsi interconnects.'),

        ('sd_health_check_delay', '10',
            'Storage domain health check delay, the amount of seconds to '
            'wait between two successive run of the domain health check.'),

        ('nfs_mount_options', 'soft,nosharecache',
            'NFS mount options, comma-separated list (NB: no white space '
            'allowed!)'),

        ('vol_extend_policy', 'ON', None),

        ('lock_util_path', '/usr/libexec/vdsm', None),

        ('lock_cmd', 'spmprotect.sh', None),

        ('free_lock_cmd', 'spmstop.sh', None),

        ('thread_pool_size', '10',
            'The number of threads to allocate to the task manager.'),

        ('max_tasks', '500', None),

        ('lvm_dev_whitelist', '', None),

        ('md_backup_versions', '30', None),

        ('md_backup_dir', '/var/log/vdsm/backup', None),

        ('maximum_allowed_pvs', '8',
            'The number of PVs per VG has a hard-coded limit of 10.'),

        ('repo_stats_cache_refresh_timeout', '300', None),

        ('task_resource_default_timeout', '120000', None),

        ('prepare_image_timeout', '600000', None),

        ('gc_blocker_force_collect_interval', '60', None),

        ('maximum_domains_in_pool', '100',
            'Process pool configuration.'),

        ('process_pool_timeout', '60', None),

        ('max_ioprocess_idle_time', '60',
            'TTL of an unused IOProcess instance'),

        ('process_pool_max_slots_per_domain', '10', None),

        ('process_pool_max_queued_slots_per_domain', '10', None),

        ('iscsi_default_ifaces', 'default',
            'Comma seperated ifaces to connect with. '
            'i.e. iser,default'),

        ('use_volume_leases', 'false',
            'Whether to use the volume leases or not.'),

        ('oop_impl', 'ioprocess',
            'Whether to use RFH or ioprocess implementation for oop.'
            'Values can be either ioprocess or rfh.'),
    ]),

    # Section: [addresses]
    ('addresses', [

        ('management_port', '54321',
            'Port on which the vdsmd XMPRPC server listens to network '
            'clients.'),

        ('json_port', '4044',
            'Port on which the vdsmd Json RPC server listens to network '
            'clients.'),

        ('management_ip', '0.0.0.0', 'Set to "::" to listen on IPv6.'),

        ('guests_gateway_ip', '', None),

    ]),
]


def set_defaults(config):
    for section, keylist in parameters:
        config.add_section(section)
        for key, value, comment in keylist:
            config.set(section, key, value)


def print_config():
    twp = textwrap.TextWrapper(initial_indent='# ', subsequent_indent='# ')

    print twp.fill("VDSM Sample Configuration")
    print

    for section, keylist in parameters:
        print "[%s]\n" % section

        for key, value, comment in keylist:
            if comment:
                print twp.fill(comment)
            print twp.fill("%s = %s" % (key, value))
            print

config = ConfigParser.ConfigParser()
set_defaults(config)
config.read([os.path.join('/etc/vdsm', 'vdsm.conf')])

if __name__ == '__main__':
    print_config()
